/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */

#include "wsensor-node-helper.h"

namespace ns3 {

//Zagotovi, da se registrira TypeId
NS_OBJECT_ENSURE_REGISTERED (WSensorNodeHelper);


TypeId
WSensorNodeHelper::GetTypeId (void)
{
  static TypeId tid =
    TypeId ("ns3::WSensorNodeHelper")
    .SetParent<Object> ()
    .AddConstructor<WSensorNodeHelper> ()
    .AddAttribute ("ListenerPort", "The port used for neighbor discovery applications.",
                   TypeId::ATTR_CONSTRUCT | TypeId::ATTR_SET | TypeId::ATTR_GET,
                   UintegerValue (0),
                   MakeUintegerAccessor (&WSensorNodeHelper::m_port),
                   MakeUintegerChecker<uint16_t> ())
    .AddAttribute ("Kernel", "Kernel size dimension kxk",
                   TypeId::ATTR_CONSTRUCT | TypeId::ATTR_SET | TypeId::ATTR_GET,
                   UintegerValue (0),
                   MakeUintegerAccessor (&WSensorNodeHelper::m_k),
                   MakeUintegerChecker<uint16_t> ())
    .AddAttribute ("PackNum", "Number of packets to send",
                   TypeId::ATTR_CONSTRUCT | TypeId::ATTR_SET | TypeId::ATTR_GET,
                   UintegerValue (0),
                   MakeUintegerAccessor (&WSensorNodeHelper::packNum),
                   MakeUintegerChecker<uint16_t> ())
    .AddAttribute ("DataRate", "Link datarate in Mbps",
                   TypeId::ATTR_CONSTRUCT | TypeId::ATTR_SET | TypeId::ATTR_GET,
                   DoubleValue (0),
                   MakeDoubleAccessor (&WSensorNodeHelper::dataRate),
                   MakeDoubleChecker<double> ())
    .AddAttribute ("PacketSize", "Packet size in bytes",
                   TypeId::ATTR_CONSTRUCT | TypeId::ATTR_SET | TypeId::ATTR_GET,
                   UintegerValue (0),
                   MakeUintegerAccessor (&WSensorNodeHelper::packetSize),
                   MakeUintegerChecker<uint16_t> ())
    .AddAttribute ("SimulationManager",
                   "Manage the simulation",
                   PointerValue (0),
                   MakePointerAccessor (&WSensorNodeHelper::m_sim_manager),
                   MakePointerChecker<WsimulationManager> ())
    .AddAttribute ("Grid",
                   "Grid rapresentation of the network",
                   PointerValue (0),
                   MakePointerAccessor (&WSensorNodeHelper::m_grid),
                   MakePointerChecker<Grid> ());
  return tid;
}

WSensorNodeHelper::WSensorNodeHelper() :
  m_port(0)
{
  m_factory.SetTypeId ("ns3::Wsensor_node");
}



WSensorNodeHelper::WSensorNodeHelper(uint16_t port, Ptr<WsimulationManager> manager, Ptr<Grid> grid, uint16_t k, uint16_t p,double dr,uint16_t ps) :
  m_port(port),
  m_sim_manager(manager),
  m_grid(grid),
  m_k(k),
  packNum(p),
  dataRate(dr),
  packetSize(ps)
{
  m_factory.SetTypeId ("ns3::Wsensor_node");
  m_factory.Set ("ListenerPort", UintegerValue (m_port));
  m_factory.Set ("Grid", PointerValue (m_grid));
  m_factory.Set ("SimulationManager", PointerValue (m_sim_manager));
  m_factory.Set ("Kernel", UintegerValue (m_k));
  m_factory.Set ("PackNum", UintegerValue (packNum));
  m_factory.Set ("DataRate", DoubleValue (dataRate));
  m_factory.Set ("PacketSize", UintegerValue (packetSize));
}


WSensorNodeHelper::~WSensorNodeHelper()
{
}


void
WSensorNodeHelper::SetAttribute (std::string name, const AttributeValue &value)
{
  m_factory.Set (name, value);
}


ApplicationContainer
WSensorNodeHelper::Install (NodeContainer c) const
{
  ApplicationContainer apps;
  for (NodeContainer::Iterator i = c.Begin (); i != c.End (); ++i)
  {
    apps.Add (InstallPriv (*i));
  }

  return apps;
}


Ptr<Application>
WSensorNodeHelper::InstallPriv (Ptr<Node> node) const
{
  Ptr<Application> app = m_factory.Create<Application> ();

  node->AddApplication (app);

  return app;
}

}

