/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */

#include "wsensor-node.h"



namespace ns3 {


//Zagotovi, da se registrira TypeId
NS_OBJECT_ENSURE_REGISTERED (Wsensor_node);

NS_LOG_COMPONENT_DEFINE("wsensornode");


TypeId
Wsensor_node::GetTypeId (void)
{
	static TypeId tid = TypeId ("ns3::Wsensor_node")
	                    .SetParent<Application> ()
	                    .AddConstructor<Wsensor_node> ()
	                    .AddAttribute ("ListenerPort",
	                                   "Port on which we listen for incoming packets.",
	                                   TypeId::ATTR_CONSTRUCT | TypeId::ATTR_SET | TypeId::ATTR_GET,
	                                   UintegerValue (80),
	                                   MakeUintegerAccessor (&Wsensor_node::m_port),
	                                   MakeUintegerChecker<uint16_t> ())
	                    .AddAttribute ("Kernel", "Kernel size dimension kxk",
	                                   TypeId::ATTR_CONSTRUCT | TypeId::ATTR_SET | TypeId::ATTR_GET,
	                                   UintegerValue (0),
	                                   MakeUintegerAccessor (&Wsensor_node::m_k),
	                                   MakeUintegerChecker<uint16_t> ())
	                    .AddAttribute ("PackNum", "Number of packets to send",
	                                   TypeId::ATTR_CONSTRUCT | TypeId::ATTR_SET | TypeId::ATTR_GET,
	                                   UintegerValue (0),
	                                   MakeUintegerAccessor (&Wsensor_node::packNum),
	                                   MakeUintegerChecker<uint16_t> ())
	                    .AddAttribute ("DataRate", "Link datarate in Mbps",
	                                   TypeId::ATTR_CONSTRUCT | TypeId::ATTR_SET | TypeId::ATTR_GET,
	                                   DoubleValue (0),
	                                   MakeDoubleAccessor (&Wsensor_node::dataRate),
	                                   MakeDoubleChecker<double> ())
	                    .AddAttribute ("PacketSize", "Packet size in bytes",
	                                   TypeId::ATTR_CONSTRUCT | TypeId::ATTR_SET | TypeId::ATTR_GET,
	                                   UintegerValue (0),
	                                   MakeUintegerAccessor (&Wsensor_node::packetSize),
	                                   MakeUintegerChecker<uint16_t> ())
	                    .AddAttribute ("SimulationManager",
	                                   "Manage the simulation",
	                                   PointerValue (0),
	                                   MakePointerAccessor (&Wsensor_node::m_sim_manager),
	                                   MakePointerChecker<WsimulationManager> ())
	                    .AddAttribute ("Grid",
	                                   "Grid rapresentation of the network",
	                                   PointerValue (0),
	                                   MakePointerAccessor (&Wsensor_node::m_grid),
	                                   MakePointerChecker<Grid> ());

	return tid;
}


Wsensor_node::Wsensor_node(): local (Ipv4Address::GetAny (), 0) {}

Wsensor_node::~Wsensor_node() {}





void
Wsensor_node::Send (int destination,WiredNode* a)
{
	NS_LOG_FUNCTION (this);
	//NS_ASSERT (m_sendEvent.IsExpired ());

	//select from which nic send the packet
	bool noDiag = m_grid->isGrid();

	NetDeviceContainer source;

	if (noDiag) {
		WiredNodeGrid* b = static_cast<WiredNodeGrid*>(a);
		source = b->getNetDevByNode(m_node);
	}else{
		source = a->getNetDevByNode(base_node);
	}

	dest_address.Set(destination);

	Ptr<Socket> snd_socket;

	if (snd_socket == 0)
	{
		TypeId tid = TypeId::LookupByName ("ns3::UdpSocketFactory");
		snd_socket = Socket::CreateSocket (GetNode (), tid);
		

		//snd_socket->BindToNetDevice (source.Get(0));
		snd_socket->Bind ();
		snd_socket->Connect (InetSocketAddress (Ipv4Address::ConvertFrom(dest_address), m_port));
		
	}


	Ptr<Packet> p = Create<Packet> (packetSize); // set also packet size

	//build header to record time
	timepacket::pHeader pTime;
	pTime.set_sentat(Simulator::Now().GetNanoSeconds());
	pTime.set_nodeid(m_id);
	pTime.set_seqnum(packNum);
	GridHeader ph(pTime);
	p->AddHeader (ph);


	//send the packet
	std::stringstream peerAddressStringStream;
	if (Ipv4Address::IsMatchingType (dest_address))
	{
		peerAddressStringStream << Ipv4Address::ConvertFrom (dest_address);
	}

	if ((snd_socket->Send (p)) >= 0)
	{
		//NS_LOG_INFO ("Packet sent to " << peerAddressStringStream.str () << Simulator::Now ().As (Time::S) << " the time" << " packet size (B): " << p->GetSize ());
		pSent++;
	}
	else
	{
		NS_LOG_INFO ("Error while sending " << peerAddressStringStream.str ());
	}

	snd_socket->Close ();

}



void
Wsensor_node::HandleRead (Ptr<Socket> socket)
{
	NS_LOG_FUNCTION (this << socket);


	Ptr<Packet> packet;
	Address from;
	Address localAddress;
	//when recieve packet, capture the receiver address
	while ((packet = socket->RecvFrom (from)))
	{
		socket->GetSockName (localAddress);
		if (InetSocketAddress::IsMatchingType (from))
		{
			//retrieve sent time
			GridHeader ph;
			packet->RemoveHeader (ph);
			timepacket::pHeader pTime = ph.GetData();
			uint64_t sent_time = pTime.sentat();
			uint32_t snd_id = pTime.nodeid();
			uint32_t p_num = pTime.seqnum();

			//only if sequence numbers are synched - otherwise the packet was delayed
			if (p_num == packNum) {
				//set value of check array
				checkRecv[indexFromId(snd_id)] = 1;

				//only if the tracked sensor
				if (m_id == trackId && Simulator::Now().GetNanoSeconds() - sent_time < 1000000000) {
					m_sim_manager->ttp(m_id, m_k, dataRate, packNum, snd_id, getLogiCoordsById(snd_id), Time(Simulator::Now().GetNanoSeconds() - sent_time));
				}
			}

			//debug
			/*if(snd_id==(uint32_t)trackId&&packNum==90){
				NS_LOG_INFO ("Node id: " << m_id << " from " << InetSocketAddress::ConvertFrom (from).GetIpv4 () << " getInt " << InetSocketAddress::ConvertFrom (from).GetIpv4 ().Get() << " port " << InetSocketAddress::ConvertFrom (from).GetPort () <<  " TTA: " <<  NanoSeconds(Simulator::Now().GetNanoSeconds() - sent_time).As (Time::S)  << " grid-id: " << snd_id);
			}*/

			//NS_LOG_INFO ("Node id: " << m_id << " Received packet at time " << Simulator::Now ().As (Time::S) << " of size " << packet->GetSize () << " bytes from " <<
			//             InetSocketAddress::ConvertFrom (from).GetIpv4 () << " port " << InetSocketAddress::ConvertFrom (from).GetPort () <<  " TTA: " <<  NanoSeconds(Simulator::Now().GetNanoSeconds() - sent_time).As (Time::S)  << " grid-id: " << snd_id);



		}

	}


}

void
Wsensor_node::SendPackets (void)
{
	//only if the tracked sensor
	if (m_id == trackId) {
		check();
	}

	silentCheck();

	//reset the check array
	resetCheck();


	if (packNum > 0) {
		m_sendEvent = Simulator::Schedule (Seconds (1), &Wsensor_node::SendPackets, this);

        //std::random_shuffle(recvList.begin(), recvList.end());

        //std::string str="";
		for (int i = 0; i < (int)recvList.size(); ++i){
		//for (auto & i : recvList) {
			int x = m_grid->getXbyID(sndList[i]);
			int y = m_grid->getYbyID(sndList[i]);
			//str = str +","+ std::to_string(recvList[i]);
			m_sendEvent = Simulator::Schedule (MilliSeconds (rnd->GetValue ()), &Wsensor_node::Send, this, recvList[i],m_grid->getNode(y, x));
		}
		//std::cout << str <<std::endl;

	}

	if (packNum != 0)
	{
		packNum--;
	}else{
		//std::cout << "id-"<<m_id << " packets sent: "<< pSent <<std::endl;
		m_sim_manager->packetsSent(pSent);
	}





}


//return the index where the id is located on the vector
int Wsensor_node::indexFromId(int id) {

	for (int i = 0; i < (int)sndList.size(); ++i)
	{
		if (sndList[i] == id) {
			return i;
		}
	}
	return 0;
}



//check to identify from which sensor node were not received packets
void Wsensor_node::check() {

	//std::string str = "";

	for (int i = 0; i < (int)checkRecv.size(); ++i)
	{
		if (checkRecv[i] == 0) {
			//str = str + std::to_string(sndList[i]) + ",";
			m_sim_manager->lost(m_id, m_k, dataRate, packNum, sndList[i], getLogiCoordsById(sndList[i]));
		}
	}
	//return str;
}



//check made by all nodes in the network
void Wsensor_node::silentCheck() {

	//std::string str = "";

	for (int i = 0; i < (int)checkRecv.size(); ++i)
	{
		if (checkRecv[i] == 0) {
			//str = str + std::to_string(sndList[i]) + ",";
			m_sim_manager->all_lost(m_id, m_k, dataRate, packNum, sndList[i], std::to_string(m_grid->getYbyID(sndList[i])) + "," + std::to_string(m_grid->getXbyID(sndList[i])));
		}
	}
	//return str;
}


//reset the check array
void Wsensor_node::resetCheck() {

	for (int i = 0; i < (int)checkRecv.size(); ++i)
	{
		checkRecv[i] = 0;
	}
}

//get logical coordinates by id of sensor node in respect to the kernel
std::string Wsensor_node::getLogiCoordsById(int id) {
	int x = m_grid->getXbyID(id) - m_x + (m_k / 2);
	int y = m_grid->getYbyID(id) - m_y + (m_k / 2);

	return std::to_string(y) + "," + std::to_string(x);

}




void
Wsensor_node::StartApplication() {
	this->Configure();

	//NS_LOG_INFO ("Starting the node " << local);



	//construct socket and listen
	if (m_socket == 0)
	{
		TypeId tid = TypeId::LookupByName ("ns3::UdpSocketFactory");
		m_socket = Socket::CreateSocket (GetNode (), tid);
		local = InetSocketAddress (Ipv4Address::GetAny (), m_port);
		if (m_socket->Bind (local) == -1)
		{
			NS_FATAL_ERROR ("Failed to bind socket");
		}
	}



	m_socket->SetRecvCallback (MakeCallback (&Wsensor_node::HandleRead, this));


	m_sendEvent = Simulator::Schedule (Seconds (1), &Wsensor_node::SendPackets, this);

}

void
Wsensor_node::StopApplication() {}


void
Wsensor_node::Configure()
{

	//get coords in the grid from the node id which is the consecutive number in the node container
	Ptr <Node> PtrNode = this->GetNode();
	//std::cout << "Node ID " << PtrNode->GetId() << " x-coord "<< m_grid->getXbyID(PtrNode->GetId()) <<" y-coord " << m_grid->getYbyID(PtrNode->GetId()) <<std::endl;

	m_id = PtrNode->GetId();
	m_x = m_grid->getXbyID(m_id);
	m_y = m_grid->getYbyID(m_id);
	base_node = m_grid->getNode(m_y, m_x);

	bool noDiag = m_grid->isGrid();

	if (noDiag) {
		m_node = static_cast<WiredNodeGrid*>(base_node);
		//std::cout << "no diag" <<std::endl;
	}else{
		base_node = dynamic_cast<WiredNode*>(base_node);
	}

	//m_grid->getNode(m_y,m_x)->printDevices();

	//std::string str = "" + std::to_string(m_id)+"----";

	//construct list of receivers and list of senders
	for (int i = -(m_k / 2); i <= (m_k / 2); ++i)
	{
		for (int j = -(m_k / 2); j <= (m_k / 2); ++j)
		{
			if (m_y + i >= 0 && m_y + i < m_grid->getN() && m_x + j >= 0 && m_x + j < m_grid->getN() && (i != 0 || j != 0)) {
				if (noDiag) {
					recvList.push_back(m_node->getAddress((WiredNodeGrid*)m_grid->getNode(m_y + i, m_x + j)));
					//str = str + ", id:"+  std::to_string(m_grid->getIDbyCoords(m_y+i,m_x+j)) +" ip:"+ std::to_string(m_node->getAddress((WiredNodeGrid*)m_grid->getNode(m_y+i,m_x+j)));
				} else {
					recvList.push_back(base_node->getAddress((WiredNode*) m_grid->getNode(m_y + i, m_x + j)));
					//str = str + ", id:"+  std::to_string(m_grid->getIDbyCoords(m_y+i,m_x+j)) +" ip:"+ std::to_string(m_node->getAddress(m_grid->getNode(m_y+i,m_x+j)));
				}
				sndList.push_back(m_grid->getIDbyCoords(m_y + i, m_x + j));
				checkRecv.push_back(1); //vector to check
			}
		}
	}

	//std::cout << str <<std::endl;

	//identify the ID of the node to be tracked
	trackId = m_grid->getTracked();


	//set datarate configuration

	double tp = (1 / ((dataRate * 1024 * 1024) / (packetSize * 8))) * 1000; //milliseconds to send a packet 1Mbps / 4096b

	int waitTime = m_k / 2;
	waitTime = (waitTime + 1) * tp;


	rnd->SetAttribute ("Min", DoubleValue (waitTime + waitTime));
	rnd->SetAttribute ("Max", DoubleValue (1000 - (waitTime + waitTime)));




	//print adjacent sensor node id -- set kernel size to 3
/*	if(trackId == m_id){
	m_grid->getNode(m_y,m_x)->printDevices();
	std::string str1 = "id:"+std::to_string(m_id)+" adj:"; 
	for (int i = 0; i < (int)sndList.size(); ++i)
	{
		str1 = str1+","+ std::to_string(sndList[i]) ;
	}
	std::cout << str1 <<std::endl;
}*/
}



}

