#include <iostream>
#include <string>


#include "ns3/bridge-helper.h"
#include "ns3/command-line.h"
#include "ns3/config.h"
#include "ns3/double.h"
#include "ns3/event-id.h"
#include "ns3/internet-stack-helper.h"
#include "ns3/ipv4-address-helper.h"
#include "ns3/ipv4-interface-container.h"
#include "ns3/ipv4-list-routing-helper.h"
#include "ns3/ipv4-static-routing-helper.h"
#include "ns3/log.h"
#include "ns3/log-macros-disabled.h"
#include "ns3/net-device-container.h"
#include "ns3/node-container.h"
#include "ns3/ns2-mobility-helper.h"
#include "ns3/nstime.h"
#include "ns3/ptr.h"
#include "ns3/simulator.h"
#include "ns3/string.h"
#include "ns3/mobility-model.h"
#include "ns3/waypoint-mobility-model.h"
#include "ns3/ssid.h"
#include "ns3/mobility-module.h"
#include "ns3/core-module.h"
#include "ns3/point-to-point-module.h"
#include "ns3/network-module.h"
#include "ns3/applications-module.h"
#include "ns3/mobility-module.h"
#include "ns3/csma-module.h"
#include "ns3/internet-module.h"
#include "ns3/netanim-module.h"
#include <time.h>
#include <cmath>

#include "ns3/wsensor-node-helper.h"
#include "ns3/wsensor-node.h"
#include "ns3/wsimulationmanager.h"
#include "ns3/grid-sn.h"
#include "ns3/wirednode.h"


#ifndef WIRED_GRID_H
#define WIRED_GRID_H

using namespace ns3;

class WiredGrid
{
public:

public:
	//constructor
	WiredGrid(int netDiam, int k, int p,int ps, double dr, std::string name);
	//destructor
	~WiredGrid();

	// Configure script parameters
	void Configure(bool verbose, bool tracing, bool animate);

	//start ruznning a simulation
	void Run(uint16_t a_topology);


private:
	//simulation settings
	unsigned int 				numNodes;
	bool 						trace;
	//maximum segment size
	uint16_t 					mss;
	std::string 				animFile = "onion-routing-animation.xml" ;  // Name of file for animation output

	//create a new simulation manager which will manage the simulation and output
	Ptr<WsimulationManager> 		manager;
	std::string 				simulation_details;
	std::string 				simulation_name;


	//Other functions
	void CreateNodes();
	void BuildDiagTopology();
	void BuildGridTopology();
	void InstallInternetStack();
	void InstallApplications();
	void InitializeGraph();
	void ExportGraph();
	void PrintNodes();

	//construct p2p links
	void linkEm(int addr);

	Ptr<Grid> 			gridOfNodes;
	WiredNode 		*first;
	WiredNode 		*second;
	int n; //network diameter
	int kernel; //only kxk kernels are allowed
	int packNum; // number of packets that each node will send
	double dataRate; //link datarate in Mbps
	int packetSize;//packet size in bytes

	//set p2p link
	Ipv4AddressHelper address;
	PointToPointHelper pointToPoint;
	NodeContainer 					nodepool;
	NetDeviceContainer 				devicepool;
	Ipv4InterfaceContainer 			interfacepool;


	MobilityHelper 					mobility;
	NodeContainer 					nodes;
	NodeContainer secundo;
	NetDeviceContainer 				netdevices;
	NetDeviceContainer 				netdevicespol;
	Ipv4InterfaceContainer 			netinterfaces;
	Ipv4InterfaceContainer 			netinterfacespol;
	ApplicationContainer 			nodeapps;




};

#endif // WIRED_GRID_H
